import requests
from bustimes import process_data

class Bus:

    def __init__(self, api_key: str):
        self.api_key = api_key

    def get_bus_stop(self, stop_id: int):
        ''' Gets bus stop info about a specific bus stop.\n
        Args:
            stop_id (int): The bus stop number.
        '''
        
        params = {'apikey': self.api_key}
        headers = {'Accept': 'application/json'}
        bus_data = requests.get(f'https://api.translink.ca/rttiapi/v1/stops/{stop_id}', params=params, headers=headers, timeout=5)
        if bus_data.status_code == 200:
            bus_data = bus_data.json()
            bus_data = process_data.process_bus_stop(bus_data)
            return bus_data
            
    def get_bus_stops_around_location(self,
                                    route: int = None, lat: float = None,
                                    long: float = None, radius: int = 500):
        ''' Gets bus stop info about all bus stops around a specific location.\n
        Args:
            route (int): (Optional) Used to filter stops by bus route.\n
            lat (float): The latitude of the location.\n
            long (float): The longitude of the location.\n
            radius (int): (Optional) The radius around the location to search for bus stops. Default is 500 meters.
        '''   
        params = {'apikey': self.api_key, 'RouteNo':route, 'lat': lat, 'long': long, 'radius': radius}
        headers = {'Accept': 'application/json'}
        bus_data = requests.get('https://api.translink.ca/rttiapi/v1/stops', params=params, headers=headers, timeout=5)
        if bus_data.status_code == 200:
            bus_data = bus_data.json()
            bus_data = process_data.process_multiple_bus_stops(bus_data)
            return bus_data
        else:
            bus_data = bus_data.json()
            if bus_data['Code'] == '10001':
                raise InvalidAPIKeyError('Invalid API Key')
            if bus_data['Code'] == '10002':
                raise DBConnectionError('Database Connection Error')
            if bus_data['Code'] == '1001':
                raise InvalidBusStopError('Invalid Stop Number')
            if bus_data['Code'] == '1002':
                raise InvalidBusStopError('Stop Not Found')
            if bus_data['Code'] == '1011':
                raise InvalidLocationError('Invalid Location')
            if bus_data['Code'] == '1012':
                raise InvalidBusStopError('Stop Not Found')
            if bus_data['Code'] == '1014':
                raise RadiusTooLargeError('Radius Too Large')
            if bus_data['Code'] == '1015':
                raise InvalidBusRouteError('Invalid Route')
            raise UnknownError(bus_data['Message'])

    def get_bus_times(self, stop_id: str, number_of_departures: int = 6):
        ''' Gets bus departure times for a specific bus stop.\n
        Args:
            stop_id (str): The bus stop number.\n
            route (str): (Optional) Used to filter departure times by bus route.\n
            number_of_departures (int): (Optional) The number of departure times to return. Default is 6.
        '''
        params = {'apikey': self.api_key, 'count': number_of_departures}
        headers = {'Accept': 'application/json'}
        bus_data = requests.get(f'https://api.translink.ca/rttiapi/v1/stops/{stop_id}/estimates', params=params, headers=headers, timeout=5)
        if bus_data.status_code == 200:
            bus_data = bus_data.json()
            bus_data = process_data.process_bus_departure_times(bus_data)
            return bus_data
        bus_data = bus_data.json()
        if bus_data['Code'] == '10001':
            raise InvalidAPIKeyError('Invalid API Key')
        if bus_data['Code'] == '10002':
            raise DBConnectionError('Database Connection Error')
        if bus_data['Code'] == '3001':
            raise InvalidBusStopError('Invalid Stop Number')
        if bus_data['Code'] == '3002':
            raise InvalidBusStopError('Stop Not Found')
        if bus_data['Code'] == '3004':
            raise InvalidBusRouteError('Invalid Route')
        raise UnknownError(bus_data['Message'])

class DBConnectionError(Exception):
    pass

class InvalidAPIKeyError(Exception):
    pass

class InvalidBusStopError(Exception):
    pass

class InvalidBusRouteError(Exception):
    pass

class InvalidLocationError(Exception):
    pass

class RadiusTooLargeError(Exception):
    pass

class UnknownError(Exception):
    pass




import string
import re

def process_bus_stop(bus_data: dict):
    stop_number = int(bus_data['StopNo'])
    stop_name: str = bus_data['Name']
    stop_name = re.sub(r'\bNB\b', 'Northbound', stop_name)
    stop_name = re.sub(r'\bSB\b', 'Southbound', stop_name)
    stop_name = re.sub(r'\bEB\b', 'Eastbound', stop_name)
    stop_name = re.sub(r'\bWB\b', 'Westbound', stop_name)
    stop_name = re.sub(r'\bFS\b', '@', stop_name)
    stop_name = re.sub(r'\bNS\b', '@', stop_name)
    stop_name = re.sub(r'\bSTN\b', 'Station', stop_name)
    stop_name = string.capwords(stop_name, sep=' ')
    bay_number: str = bus_data['BayNo']
    if bay_number == 'N':
        bay_number = None
    else:
        bay_number = int(bay_number)
    at_street: str = bus_data['AtStreet']
    if re.match(r'BAY \d+', at_street):
        at_street = None
    else:
        at_street = string.capwords(at_street, sep=' ')
    routes: str = bus_data['Routes'].lstrip('0')
    return {'StopNumber': stop_number,'StopName': stop_name, 'BayNumber': bay_number, 'AtStreet': at_street, 'Routes': routes}

def process_multiple_bus_stops(bus_data: dict):
    bus_stops = []
    for bus_stop in bus_data:
        bus_stop_data = process_bus_stop(bus_stop)
        if 'Platform' in bus_stop_data['StopName'] or bus_stop_data['StopName'].endswith('New'):
            continue
        bus_stops.append(bus_stop_data)
    return bus_stops

def process_bus_departure_times(bus_data: dict):
    departure_times = {}
    for routes in bus_data:
        route_number_without_leading_zero = routes['RouteNo'].lstrip('0')
        departure_times.update({route_number_without_leading_zero:[]})
        for departure in routes['Schedules']:
            real_time: bool = False
            is_delayed: bool = False
            is_early: bool = False
            destination: str = departure['Destination']
            destination = re.sub(r'\bSTN\b', 'Station', destination)
            destination = re.sub(r'\bEXP\b', 'Express', destination)
            destination = re.sub(r'\bEXCH\b', 'Exchange', destination)
            destination = re.sub(r'\bCTRL\b', 'Central', destination)
            destination = re.sub(r'\bCTR\b', 'Centre', destination)
            destination = string.capwords(destination, sep=' ')
            leave_time: str = departure['ExpectedLeaveTime']
            leave_time = leave_time.split(' ')[0]
            status: str = departure['ScheduleStatus']
            if status == ' ':
                real_time = True
            if status == '-':
                real_time = True
                is_delayed = True
            if status == '+':
                real_time = True
                is_early = True
            if departure['CancelledTrip'] or departure['CancelledStop']:
                cancelled_trip = True
            else:
                cancelled_trip = False
            countdown_time: int = departure['ExpectedCountdown']
            departure_times[route_number_without_leading_zero].append({"RouteNumber": route_number_without_leading_zero, "Destination": destination, "LeaveTime": leave_time, "CountdownTime": countdown_time, "RealTime": real_time, "IsDelayed": is_delayed, "IsEarly": is_early, "CancelledTrip": cancelled_trip})
    return departure_times

